@@ -1,11 +1,13 @@
 import argparse
 import datetime
-from pinecone import Pinecone, Vector
 import os
 import json
 import numpy as np
 from tqdm import tqdm
 
+from pinecone import Pinecone, Vector
+from vdf_io.constants import ID_COLUMN
+
 from vdf_io.names import DBNames
 from vdf_io.meta_types import NamespaceMeta, VDFMeta
 from vdf_io.util import set_arg_from_input, set_arg_from_password, standardize_metric
@@ -17,102 +19,105 @@ MAX_FETCH_SIZE = 1_000
 THREAD_POOL_SIZE = 30
 
 
-def make_pinecone_parser(subparsers):
-    parser_pinecone = subparsers.add_parser(
-        "pinecone", help="Export data from Pinecone"
-    )
-    parser_pinecone.add_argument(
-        "-e", "--environment", type=str, help="Environment of Pinecone instance"
-    )
-    parser_pinecone.add_argument(
-        "-i", "--index", type=str, help="Name of index to export"
-    )
-    parser_pinecone.add_argument(
-        "-s", "--id_range_start", type=int, help="Start of id range", default=None
-    )
-    parser_pinecone.add_argument(
-        "--id_range_end", type=int, help="End of id range", default=None
-    )
-    parser_pinecone.add_argument(
-        "-f", "--id_list_file", type=str, help="Path to id list file", default=None
-    )
-    parser_pinecone.add_argument(
-        "--modify_to_search",
-        type=bool,
-        help="Allow modifying data to search",
-        default=False,
-        action=argparse.BooleanOptionalAction,
-    )
-    parser_pinecone.add_argument(
-        "--subset",
-        type=bool,
-        help="Export a subset of data (default: False)",
-        default=False,
-        action=argparse.BooleanOptionalAction,
-    )
-    parser_pinecone.add_argument(
-        "--namespaces",
-        type=str,
-        help="Name of namespace(s) to export (comma-separated)",
-        default=None,
-    )
-
-
-def export_pinecone(args):
-    """
-    Export data from Pinecone
-    """
-    set_arg_from_input(
-        args, "environment", "Enter the environment of Pinecone instance: "
-    )
-    set_arg_from_input(
-        args,
-        "index",
-        "Enter the name of index to export (hit return to export all): ",
-    )
-    set_arg_from_password(
-        args, "pinecone_api_key", "Enter your Pinecone API key: ", "PINECONE_API_KEY"
-    )
-    set_arg_from_input(
-        args,
-        "modify_to_search",
-        "Allow modifying data to search, enter Y or N: ",
-        bool,
-    )
-    set_arg_from_input(
-        args,
-        "namespaces",
-        "Enter the name of namespace(s) to export (comma-separated) (hit return to export all):",
-        str,
-    )
-    if args["subset"] is True:
-        if "id_list_file" not in args or args["id_list_file"] is None:
-            set_arg_from_input(
-                args,
-                "id_range_start",
-                "Enter the start of id range (hit return to skip): ",
-                int,
-            )
-            set_arg_from_input(
-                args,
-                "id_range_end",
-                "Enter the end of id range (hit return to skip): ",
-                int,
-            )
-        if args["id_range_start"] is None and args["id_range_end"] is None:
-            set_arg_from_input(
-                args,
-                "id_list_file",
-                "Enter the path to id list file (hit return to skip): ",
-            )
-    pinecone_export = ExportPinecone(args)
-    pinecone_export.get_data()
-    return pinecone_export
-
-
 class ExportPinecone(ExportVDB):
     DB_NAME_SLUG = DBNames.PINECONE
 
+    @classmethod
+    def make_parser(cls, subparsers):
+        parser_pinecone = subparsers.add_parser(
+            "pinecone", help="Export data from Pinecone"
+        )
+        parser_pinecone.add_argument(
+            "-e", "--environment", type=str, help="Environment of Pinecone instance"
+        )
+        parser_pinecone.add_argument(
+            "-i", "--index", type=str, help="Name of index to export"
+        )
+        parser_pinecone.add_argument(
+            "-s", "--id_range_start", type=int, help="Start of id range", default=None
+        )
+        parser_pinecone.add_argument(
+            "--id_range_end", type=int, help="End of id range", default=None
+        )
+        parser_pinecone.add_argument(
+            "-f", "--id_list_file", type=str, help="Path to id list file", default=None
+        )
+        parser_pinecone.add_argument(
+            "--modify_to_search",
+            type=bool,
+            help="Allow modifying data to search",
+            default=False,
+            action=argparse.BooleanOptionalAction,
+        )
+        parser_pinecone.add_argument(
+            "--subset",
+            type=bool,
+            help="Export a subset of data (default: False)",
+            default=False,
+            action=argparse.BooleanOptionalAction,
+        )
+        parser_pinecone.add_argument(
+            "--namespaces",
+            type=str,
+            help="Name of namespace(s) to export (comma-separated)",
+            default=None,
+        )
+
+    @classmethod
+    def export_vdb(cls, args):
+        """
+        Export data from Pinecone
+        """
+        set_arg_from_input(
+            args, "environment", "Enter the environment of Pinecone instance: "
+        )
+        set_arg_from_input(
+            args,
+            "index",
+            "Enter the name of index to export (hit return to export all): ",
+        )
+        set_arg_from_password(
+            args,
+            "pinecone_api_key",
+            "Enter your Pinecone API key: ",
+            "PINECONE_API_KEY",
+        )
+        set_arg_from_input(
+            args,
+            "modify_to_search",
+            "Allow modifying data to search, enter Y or N: ",
+            bool,
+        )
+        set_arg_from_input(
+            args,
+            "namespaces",
+            "Enter the name of namespace(s) to export (comma-separated) (hit return to export all):",
+            str,
+        )
+        if args["subset"] is True:
+            if "id_list_file" not in args or args["id_list_file"] is None:
+                set_arg_from_input(
+                    args,
+                    "id_range_start",
+                    "Enter the start of id range (hit return to skip): ",
+                    int,
+                )
+                set_arg_from_input(
+                    args,
+                    "id_range_end",
+                    "Enter the end of id range (hit return to skip): ",
+                    int,
+                )
+            if args["id_range_start"] is None and args["id_range_end"] is None:
+                set_arg_from_input(
+                    args,
+                    "id_list_file",
+                    "Enter the path to id list file (hit return to skip): ",
+                )
+        pinecone_export = ExportPinecone(args)
+        pinecone_export.get_data()
+        return pinecone_export
+
     def __init__(self, args):
         """
         Initialize the class
@@ -138,7 +143,7 @@ class ExportPinecone(ExportVDB):
                 tqdm.write("No vectors found that have not been exported yet.")
                 return []
             # mark the vectors as exported
-            ids = [result["id"] for result in results["matches"]]
+            ids = [result[ID_COLUMN] for result in results["matches"]]
             ids_to_mark = list(set(ids) - all_ids)
             tqdm.write(
                 f"Found {len(ids_to_mark)} vectors that have not been exported yet."
@@ -199,7 +204,7 @@ class ExportPinecone(ExportVDB):
                 top_k=PINECONE_MAX_K,
                 namespace=namespace,
             )
-        ids = set(result["id"] for result in results["matches"])
+        ids = set(result[ID_COLUMN] for result in results["matches"])
         return ids
 
     def get_all_ids_from_index(self, num_dimensions, namespace="", hash_value=""):
@@ -232,7 +237,7 @@ class ExportPinecone(ExportVDB):
             top_k=100,
             namespace=namespace,
         )
-        random_results_ids_strs = [x["id"] for x in random_results["matches"]]
+        random_results_ids_strs = [x[ID_COLUMN] for x in random_results["matches"]]
         all_ids = set()
         if not all(x.isdigit() for x in random_results_ids_strs):
             tqdm.write(
@@ -436,10 +441,6 @@ class ExportPinecone(ExportVDB):
         index_metas = {}
         for index_name in tqdm(index_names, desc="Fetching indexes"):
             index_meta = self.get_data_for_index(index_name)
-            for index_meta_elem in index_meta:
-                index_meta_elem["metric"] = standardize_metric(
-                    self.pc.describe_index(index_name).metric, self.DB_NAME_SLUG
-                )
             index_metas[index_name] = index_meta
 
         # Create and save internal metadata JSON
@@ -455,7 +456,7 @@ class ExportPinecone(ExportVDB):
         vdf_meta_text = json.dumps(internal_metadata.dict(), indent=4)
         tqdm.write(vdf_meta_text)
         with open(os.path.join(self.vdf_directory, "VDF_META.json"), "w") as json_file:
-            json.dump(vdf_meta_text, json_file, indent=4)
+            json_file.write(vdf_meta_text)
         return True
 
     def get_data_for_index(self, index_name):
@@ -541,6 +542,10 @@ class ExportPinecone(ExportVDB):
                 model_name=self.args["model_name"],
                 vector_columns=["vector"],
                 data_path="/".join(vectors_directory.split("/")[1:]),
+                metric=standardize_metric(
+                    self.pc.describe_index(index_name).metric, self.DB_NAME_SLUG
+                ),
             )
             index_meta.append(namespace_meta)
+            self.args["exported_count"] += total_size
         return index_meta
